@@ -2,6 +2,7 @@
  * File: file.c :)
  *
  * Copyright (C) 2000-2007 Jorge Arellano Cid <jcid@dillo.org>
+ * Copyright (C) 2024 Rodrigo Arias Mallo <rodarima@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -138,7 +139,7 @@ static const char *File_get_content_type_from_data(void *Data, size_t Size)
    char *p = Data;
    size_t i, non_ascci;
 
-   _MSG("File_get_content_type_from_data:: Size = %d\n", Size);
+   _MSG("File_get_content_type_from_data:: Size = %zu\n", Size);
 
    /* HTML try */
    for (i = 0; i < Size && dIsspace(p[i]); ++i);
@@ -353,7 +354,7 @@ static void File_info2html(ClientInfo *client, FileInfo *finfo, int n)
    }
 
    /* we could note if it's a symlink... */
-   if S_ISDIR (finfo->mode) {
+   if (S_ISDIR (finfo->mode)) {
       filecont = "Directory";
    } else if (finfo->mode & (S_IXUSR | S_IXGRP | S_IXOTH)) {
       filecont = "Executable";
@@ -510,6 +511,7 @@ static const char *File_ext(const char *filename)
    } else if (!dStrAsciiCasecmp(e, "png")) {
       return "image/png";
    } else if (!dStrAsciiCasecmp(e, "html") ||
+              !dStrAsciiCasecmp(e, "xhtml") ||
               !dStrAsciiCasecmp(e, "htm") ||
               !dStrAsciiCasecmp(e, "shtml")) {
       return "text/html";
@@ -790,6 +792,7 @@ static int File_parse_hex_octet(const char *s)
 
 /*
  * Make a file URL into a human (and machine) readable path.
+ * The tilde '~' character is expanded to the value of the user home.
  * The idea is to always have a path that starts with only one slash.
  * Embedded slashes are ignored.
  */
@@ -797,16 +800,30 @@ static char *File_normalize_path(const char *orig)
 {
    char *str = (char *) orig, *basename = NULL, *ret = NULL, *p;
 
-   dReturn_val_if (orig == NULL, ret);
+   if (str == NULL)
+      return NULL;
 
-   /* Make sure the string starts with "file:/" */
-   if (dStrnAsciiCasecmp(str, "file:/", 5) != 0)
+   /* Make sure the string starts with "file:" */
+   if (dStrnAsciiCasecmp(str, "file:", 5))
       return ret;
-   str += 5;
 
-   /* Skip "localhost" */
-   if (dStrnAsciiCasecmp(str, "//localhost/", 12) == 0)
+   str += 5; /* skip "file:" */
+
+   char *tmp = NULL;
+
+   if (str[0] == '~' && (str[1] == '/' || str[1] == '\0')) {
+      /* Expand home tilde "~" into "/home/userxyz" */
+      const char *home = dGethomedir();
+      /* Add separator if needed */
+      char *sep = home[strlen(home) - 1] == '/' ? "" : "/";
+      char *next = str + 1;
+      while (*next == '/')
+         next++;
+      str = tmp = dStrconcat(home, sep, next, NULL);
+   } else if (dStrnAsciiCasecmp(str, "//localhost/", 12) == 0) {
+      /* Skip "//localhost" */
       str += 11;
+   }
 
    /* Skip packed slashes, and leave just one */
    while (str[0] == '/' && str[1] == '/')
@@ -820,6 +837,8 @@ static char *File_normalize_path(const char *orig)
                    basename ? "/" : "",
                    str);
       dFree(basename);
+      if (tmp)
+         dFree(tmp);
 
       /* Parse possible hexadecimal octets in the URI path */
       for (i = 0; ds->str[i]; ++i) {
