@@ -18,7 +18,6 @@
  */
 #define _MSG(...)
 #define MSG(...)  printf("[dpid]: " __VA_ARGS__)
-#define _MSG_ERR(...)
 #define MSG_ERR(...)  fprintf(stderr, "[dpid]: " __VA_ARGS__)
 
 #define dotDILLO_DPI ".dillo/dpi"
@@ -36,13 +35,6 @@
 #define CKD_WRITE(fd, msg) ckd_write(fd, msg, __FILE__, __LINE__)
 #define CKD_CLOSE(fd)      ckd_close(fd, __FILE__, __LINE__)
 
-
-/*! Error codes for dpid */
-enum {
-   no_errors,
-   dpid_srs_addrinuse /* dpid service request socket address already in use */
-} dpi_errno;
-
 /*! Intended for identifying dillo plugins
  * and related files
  */
