@@ -96,7 +96,7 @@ FindtextState::Result FindtextState::search (const char *key, bool caseSens,
 
       if (iterator)
          delete iterator;
-      iterator = new CharIterator (widget);
+      iterator = new CharIterator (widget, true);
 
       if (backwards) {
          /* Go to end */
@@ -128,7 +128,7 @@ FindtextState::Result FindtextState::search (const char *key, bool caseSens,
       } else {
          // Nothing found anymore, reset the state for the next trial.
          delete iterator;
-         iterator = new CharIterator (widget);
+         iterator = new CharIterator (widget, true);
          if (backwards) {
             /* Go to end */
             while (iterator->next ()) ;
@@ -221,7 +221,7 @@ bool FindtextState::unhighlight ()
       return false;
 }
 
-bool FindtextState::search0 (bool backwards,  bool firstTrial)
+bool FindtextState::search0 (bool backwards, bool firstTrial)
 {
    if (iterator->getChar () == CharIterator::END)
       return false;
