@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-
-
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
@@ -75,6 +73,12 @@ void StyleAttrs::initValues ()
    backgroundPositionX = createPerLength (0);
    backgroundPositionY = createPerLength (0);
    width = height = lineHeight = LENGTH_AUTO;
+   minWidth = maxWidth = minHeight = maxHeight = LENGTH_AUTO;
+   vloat = FLOAT_NONE;
+   clear = CLEAR_NONE;
+   overflow = OVERFLOW_VISIBLE;
+   position = POSITION_STATIC;
+   top = bottom = left = right = LENGTH_AUTO;
    textIndent = 0;
    margin.setVal (0);
    borderWidth.setVal (0);
@@ -89,6 +93,7 @@ void StyleAttrs::initValues ()
    display = DISPLAY_INLINE;
    whiteSpace = WHITE_SPACE_NORMAL;
    cursor = CURSOR_DEFAULT;
+   zIndex = Z_INDEX_AUTO;
 }
 
 /**
@@ -101,6 +106,12 @@ void StyleAttrs::resetValues ()
 
    valign = VALIGN_BASELINE;
    textAlignChar = '.';
+   vloat = FLOAT_NONE; /** \todo Correct? Check specification. */
+   clear = CLEAR_NONE; /** \todo Correct? Check specification. */
+   overflow = OVERFLOW_VISIBLE;
+   position = POSITION_STATIC; /** \todo Correct? Check specification. */
+   top = bottom = left = right = LENGTH_AUTO; /** \todo Correct? Check
+                                                  specification. */
    backgroundColor = NULL;
    backgroundImage = NULL;
    backgroundRepeat = BACKGROUND_REPEAT;
@@ -109,6 +120,7 @@ void StyleAttrs::resetValues ()
    backgroundPositionY = createPerLength (0);
    width = LENGTH_AUTO;
    height = LENGTH_AUTO;
+   minWidth = maxWidth = minHeight = maxHeight = LENGTH_AUTO;
 
    margin.setVal (0);
    borderWidth.setVal (0);
@@ -156,11 +168,23 @@ bool StyleAttrs::equals (object::Object *other) {
        valign == otherAttrs->valign &&
        textAlignChar == otherAttrs->textAlignChar &&
        textTransform == otherAttrs->textTransform &&
+       vloat == otherAttrs->vloat &&
+       clear == otherAttrs->clear &&
+       overflow == otherAttrs->overflow &&
+       position == otherAttrs->position &&
+       top == otherAttrs->top &&
+       bottom == otherAttrs->bottom &&
+       left == otherAttrs->left &&
+       right == otherAttrs->right &&
        hBorderSpacing == otherAttrs->hBorderSpacing &&
        vBorderSpacing == otherAttrs->vBorderSpacing &&
        wordSpacing == otherAttrs->wordSpacing &&
        width == otherAttrs->width &&
        height == otherAttrs->height &&
+       minWidth == otherAttrs->minWidth &&
+       maxWidth == otherAttrs->maxWidth &&
+       minHeight == otherAttrs->minHeight &&
+       maxHeight == otherAttrs->maxHeight &&
        lineHeight == otherAttrs->lineHeight &&
        textIndent == otherAttrs->textIndent &&
        margin.equals (&otherAttrs->margin) &&
@@ -180,6 +204,7 @@ bool StyleAttrs::equals (object::Object *other) {
        listStylePosition == otherAttrs->listStylePosition &&
        listStyleType == otherAttrs->listStyleType &&
        cursor == otherAttrs->cursor &&
+       zIndex == otherAttrs->zIndex &&
        x_link == otherAttrs->x_link &&
        x_lang[0] == otherAttrs->x_lang[0] &&
        x_lang[1] == otherAttrs->x_lang[1] &&
@@ -201,11 +226,23 @@ int StyleAttrs::hashValue () {
       valign +
       textAlignChar +
       textTransform +
+      vloat +
+      clear +
+      overflow +
+      position +
+      top +
+      bottom +
+      left +
+      right +
       hBorderSpacing +
       vBorderSpacing +
       wordSpacing +
       width +
       height +
+      minWidth +
+      maxWidth +
+      minHeight +
+      maxHeight +
       lineHeight +
       textIndent +
       margin.hashValue () +
@@ -225,6 +262,7 @@ int StyleAttrs::hashValue () {
       listStylePosition +
       listStyleType +
       cursor +
+      zIndex +
       x_link +
       x_lang[0] + x_lang[1] +
       x_img +
@@ -316,6 +354,14 @@ void Style::copyAttrs (StyleAttrs *attrs)
    valign = attrs->valign;
    textAlignChar = attrs->textAlignChar;
    textTransform = attrs->textTransform;
+   vloat = attrs->vloat;
+   clear = attrs->clear;
+   overflow = attrs->overflow;
+   position = attrs->position;
+   top = attrs->top;
+   bottom = attrs->bottom;
+   left = attrs->left;
+   right = attrs->right;
    hBorderSpacing = attrs->hBorderSpacing;
    vBorderSpacing = attrs->vBorderSpacing;
    wordSpacing = attrs->wordSpacing;
@@ -323,6 +369,10 @@ void Style::copyAttrs (StyleAttrs *attrs)
    height = attrs->height;
    lineHeight = attrs->lineHeight;
    textIndent = attrs->textIndent;
+   minWidth = attrs->minWidth;
+   maxWidth = attrs->maxWidth;
+   minHeight = attrs->minHeight;
+   maxHeight = attrs->maxHeight;
    margin = attrs->margin;
    borderWidth = attrs->borderWidth;
    padding = attrs->padding;
@@ -334,6 +384,7 @@ void Style::copyAttrs (StyleAttrs *attrs)
    listStylePosition = attrs->listStylePosition;
    listStyleType = attrs->listStyleType;
    cursor = attrs->cursor;
+   zIndex = attrs->zIndex;
    x_link = attrs->x_link;
    x_lang[0] = attrs->x_lang[0];
    x_lang[1] = attrs->x_lang[1];
@@ -701,7 +752,7 @@ Length StyleImage::ExternalWidgetImgRenderer::getBackgroundPositionY ()
  * use a trapezium as draw polygon, or drawTypedLine() for dots and dashes.
  * Although the concept is simple, achieving pixel accuracy is laborious [1].
  *
- * [1] http://www.dillo.org/css_compat/tests/border-style.html
+ * [1] https://dillo-browser.github.io/old/css_compat/tests/border-style.html
  */
 static void drawBorderTop(View *view, Style *style,
                           int x1, int y1, int x2, int y2)
@@ -1157,18 +1208,26 @@ void drawBorder (View *view, Layout *layout, Rectangle *area,
  *
  * Otherwise, the caller should not try to increase the performance by
  * doing some tests before; this is all done in this method.
+ * 
+ * "bgColor" is passes implicitly. For non-inversed drawing,
+ * style->backgroundColor may simply used. However, when drawing is
+ * inversed, and style->backgroundColor is undefined (NULL), a
+ * background color defined higher in the hierarchy (which is not
+ * accessable here) must be used.
+ *
+ * (Background *images* are never drawn inverse.)
  */
 void drawBackground (View *view, Layout *layout, Rectangle *area,
                      int x, int y, int width, int height,
                      int xRef, int yRef, int widthRef, int heightRef,
-                     Style *style, bool inverse, bool atTop)
+                     Style *style, Color *bgColor, bool inverse, bool atTop)
 {
-   bool bgColor = style->backgroundColor != NULL &&
+   bool hasBgColor = bgColor != NULL &&
       // The test for background colors is rather simple, since only the color
       // has to be compared, ...
-      (!atTop || layout->getBgColor () != style->backgroundColor);
-   bool bgImage = (style->backgroundImage != NULL &&
-                   style->backgroundImage->getImgbufSrc() != NULL) &&
+      (!atTop || layout->getBgColor () != bgColor);
+   bool hasBgImage = (style->backgroundImage != NULL &&
+                      style->backgroundImage->getImgbufSrc() != NULL) &&
       // ... but for backgrounds, it would be rather complicated. To handle the
       // two cases (normal HTML in a viewport, where the layout background
       // image is set, and contents of <button> within a flat view, where the
@@ -1182,7 +1241,7 @@ void drawBackground (View *view, Layout *layout, Rectangle *area,
    // necessary to draw the background if background color and image
    // are not set (NULL), i. e. shining through.
 
-   if (bgColor || bgImage) {
+   if (hasBgColor || hasBgImage) {
       Rectangle bgArea, intersection;
       bgArea.x = x;
       bgArea.y = y;
@@ -1190,14 +1249,14 @@ void drawBackground (View *view, Layout *layout, Rectangle *area,
       bgArea.height = height;
 
       if (area->intersectsWith (&bgArea, &intersection)) {
-         if (bgColor)
-            view->drawRectangle (style->backgroundColor,
+         if (hasBgColor)
+            view->drawRectangle (bgColor,
                                  inverse ?
                                  Color::SHADING_INVERSE : Color::SHADING_NORMAL,
                                  true, intersection.x, intersection.y,
                                  intersection.width, intersection.height);
 
-         if (bgImage)
+         if (hasBgImage)
             drawBackgroundImage (view, style->backgroundImage,
                                  style->backgroundRepeat,
                                  style->backgroundAttachment,
