@@ -133,6 +133,7 @@ void StringBuffer::clear ()
 
 BitSet::BitSet(int initBits)
 {
+   numBits = initBits;
    numBytes = bytesForBits(initBits);
    bits = (unsigned char*)malloc(numBytes * sizeof(unsigned char));
    clear();
@@ -146,7 +147,7 @@ BitSet::~BitSet()
 void BitSet::intoStringBuffer(misc::StringBuffer *sb)
 {
    sb->append("[");
-   for (int i = 0; i < numBytes; i++)
+   for (int i = 0; i < numBits; i++)
       sb->append(get(i) ? "1" : "0");
    sb->append("]");
 }
@@ -161,12 +162,18 @@ bool BitSet::get(int i) const
 
 void BitSet::set(int i, bool val)
 {
+   if (i > numBits)
+      numBits = i;
+
    if (8 * i >= numBytes) {
       int newNumBytes = numBytes;
       while (8 * i >= newNumBytes)
          newNumBytes *= 2;
-      bits =
-         (unsigned char*)realloc(bits, newNumBytes * sizeof(unsigned char));
+
+      void *vp;
+      assert((vp = realloc(bits, newNumBytes * sizeof(unsigned char))));
+      if (!vp) exit(-2); // when NDEBUG is defined
+      bits = (unsigned char*)vp;
       memset(bits + numBytes, 0, newNumBytes - numBytes);
       numBytes = newNumBytes;
    }
