@@ -13,12 +13,6 @@
 #define IOClose  2
 #define IOAbort  3
 
-/*
- * IO Flags
- */
-#define IOFlag_ForceClose  (1 << 1)
-#define IOFlag_SingleWrite (1 << 2)
-
 /*
  * IO constants
  */
