@@ -20,10 +20,11 @@
 
 static const int bufsize = 8*1024;
 
-/*
- * Decode chunked data
+/**
+ * Decode 'Transfer-Encoding: chunked' data
  */
-static Dstr *Decode_chunked(Decode *dc, const char *instr, int inlen)
+Dstr *a_Decode_transfer_process(DecodeTransfer *dc, const char *instr,
+                                int inlen)
 {
    char *inputPtr, *eol;
    int inputRemaining;
@@ -66,6 +67,7 @@ static Dstr *Decode_chunked(Decode *dc, const char *instr, int inlen)
       }
 
       if (!(chunkRemaining = strtol(inputPtr, NULL, 0x10))) {
+         dc->finished = TRUE;
          break;   /* A chunk length of 0 means we're done! */
       }
       inputRemaining -= (eol - inputPtr) + 1;
@@ -80,10 +82,16 @@ static Dstr *Decode_chunked(Decode *dc, const char *instr, int inlen)
    return output;
 }
 
-static void Decode_chunked_free(Decode *dc)
+bool_t a_Decode_transfer_finished(DecodeTransfer *dc)
+{
+   return dc->finished;
+}
+
+void a_Decode_transfer_free(DecodeTransfer *dc)
 {
    dFree(dc->state);
    dStr_free(dc->leftover, 1);
+   dFree(dc);
 }
 
 static void Decode_compression_free(Decode *dc)
@@ -100,7 +108,7 @@ static void Decode_compression_free(Decode *dc)
  * for what it accomplished.
  */
 
-/*
+/**
  * Decode gzipped data
  */
 static Dstr *Decode_gzip(Decode *dc, const char *instr, int inlen)
@@ -136,7 +144,7 @@ static Dstr *Decode_gzip(Decode *dc, const char *instr, int inlen)
    return output;
 }
 
-/*
+/**
  * Decode (raw) deflated data
  */
 static Dstr *Decode_raw_deflate(Decode *dc, const char *instr, int inlen)
@@ -172,7 +180,7 @@ static Dstr *Decode_raw_deflate(Decode *dc, const char *instr, int inlen)
    return output;
 }
 
-/*
+/**
  * Decode deflated data, initially presuming that the required zlib wrapper
  * is there. On data error, switch to Decode_raw_deflate().
  */
@@ -208,7 +216,7 @@ static Dstr *Decode_deflate(Decode *dc, const char *instr, int inlen)
          dStr_free(output, 1);
          (void)inflateEnd(zs);
          dFree(dc->state);
-         dc->state = zs = dNew(z_stream, 1);;
+         dc->state = zs = dNew(z_stream, 1);
          zs->zalloc = NULL;
          zs->zfree = NULL;
          zs->next_in = NULL;
@@ -224,7 +232,7 @@ static Dstr *Decode_deflate(Decode *dc, const char *instr, int inlen)
    return output;
 }
 
-/*
+/**
  * Translate to desired character set (UTF-8)
  */
 static Dstr *Decode_charset(Decode *dc, const char *instr, int inlen)
@@ -277,22 +285,20 @@ static void Decode_charset_free(Decode *dc)
    dStr_free(dc->leftover, 1);
 }
 
-/*
+/**
  * Initialize transfer decoder. Currently handles "chunked".
  */
-Decode *a_Decode_transfer_init(const char *format)
+DecodeTransfer *a_Decode_transfer_init(const char *format)
 {
-   Decode *dc = NULL;
+   DecodeTransfer *dc = NULL;
 
    if (format && !dStrAsciiCasecmp(format, "chunked")) {
       int *chunk_remaining = dNew(int, 1);
       *chunk_remaining = 0;
-      dc = dNew(Decode, 1);
+      dc = dNew(DecodeTransfer, 1);
       dc->leftover = dStr_new("");
       dc->state = chunk_remaining;
-      dc->decode = Decode_chunked;
-      dc->free = Decode_chunked_free;
-      dc->buffer = NULL; /* not used */
+      dc->finished = FALSE;
       _MSG("chunked!\n");
    }
    return dc;
@@ -315,7 +321,7 @@ static Decode *Decode_content_init_common()
    return dc;
 }
 
-/*
+/**
  * Initialize content decoder. Currently handles 'gzip' and 'deflate'.
  */
 Decode *a_Decode_content_init(const char *format)
@@ -335,7 +341,7 @@ Decode *a_Decode_content_init(const char *format)
 
          dc->decode = Decode_gzip;
       } else if (!dStrAsciiCasecmp(format, "deflate")) {
-         MSG("deflated data!\n");
+         _MSG("deflated data!\n");
 
          dc = Decode_content_init_common();
          zs = (z_stream *)dc->state;
@@ -349,7 +355,7 @@ Decode *a_Decode_content_init(const char *format)
    return dc;
 }
 
-/*
+/**
  * Initialize decoder to translate from any character set known to iconv()
  * to UTF-8.
  *
@@ -380,7 +386,7 @@ Decode *a_Decode_charset_init(const char *format)
    return dc;
 }
 
-/*
+/**
  * Decode data.
  */
 Dstr *a_Decode_process(Decode *dc, const char *instr, int inlen)
