@@ -9,7 +9,9 @@
  * (at your option) any later version.
  */
 
-// UI dialogs
+/** @file
+ * UI dialogs
+ */
 
 #include <math.h> // for rint()
 
@@ -44,8 +46,8 @@ static int choice_answer;
  */
 
 //----------------------------------------------------------------------------
-/*
- * Used to enable CTRL+{a,e,d,k} in search dialog (for start,end,del,cut)
+/**
+ * Used to enable CTRL+{a,e,d,k} in search dialog (for start,end,del,cut).
  * TODO: bind down arrow to a search engine selection list.
  */
 class CustInput3 : public Fl_Input {
@@ -79,7 +81,7 @@ int CustInput3::handle(int e)
    return Fl_Input::handle(e);
 }
 
-/*
+/**
  * Used to make the ENTER key activate the CustChoice
  */
 class CustChoice2 : public Fl_Choice {
@@ -117,7 +119,7 @@ int EnterButton::handle(int e)
 //----------------------------------------------------------------------------
 
 
-/*
+/**
  * Display a message in a popup window.
  */
 void a_Dialog_msg(const char *title, const char *msg)
@@ -129,7 +131,7 @@ void a_Dialog_msg(const char *title, const char *msg)
 }
 
 
-/*
+/**
  * Callback for a_Dialog_input()
  */
 static void input_cb(Fl_Widget *button, void *number)
@@ -138,11 +140,11 @@ static void input_cb(Fl_Widget *button, void *number)
   button->window()->hide();
 }
 
-/*
+/**
  * Dialog for one line of Input with a message.
  * avoids the sound bell in fl_input(), and allows customization
  *
- * Return value: string on success, NULL upon Cancel or Close window
+ * @return string on success, NULL upon Cancel or Close window
  */
 const char *a_Dialog_input(const char *title, const char *msg)
 {
@@ -223,7 +225,7 @@ const char *a_Dialog_input(const char *title, const char *msg)
    return (input_answer == 1) ? input_str : NULL;
 }
 
-/*
+/**
  * Dialog for password
  */
 const char *a_Dialog_passwd(const char *title, const char *msg)
@@ -234,10 +236,10 @@ const char *a_Dialog_passwd(const char *title, const char *msg)
    return fl_password("%s", "", msg);
 }
 
-/*
+/**
  * Show the save file dialog.
  *
- * Return: pointer to chosen filename, or NULL on Cancel.
+ * @return pointer to chosen filename, or NULL on Cancel.
  */
 const char *a_Dialog_save_file(const char *title,
                                const char *pattern, const char *fname)
@@ -245,10 +247,10 @@ const char *a_Dialog_save_file(const char *title,
    return fl_file_chooser(title, pattern, fname);
 }
 
-/*
+/**
  * Show the select file dialog.
  *
- * Return: pointer to chosen filename, or NULL on Cancel.
+ * @return pointer to chosen filename, or NULL on Cancel.
  */
 const char *a_Dialog_select_file(const char *title,
                                  const char *pattern, const char *fname)
@@ -260,10 +262,10 @@ const char *a_Dialog_select_file(const char *title,
    return a_Dialog_save_file(title, pattern, fname);
 }
 
-/*
+/**
  * Show the open file dialog.
  *
- * Return: pointer to chosen filename, or NULL on Cancel.
+ * @return pointer to chosen filename, or NULL on Cancel.
  */
 char *a_Dialog_open_file(const char *title,
                          const char *pattern, const char *fname)
@@ -274,7 +276,7 @@ char *a_Dialog_open_file(const char *title,
    return (fc_name) ? a_Misc_escape_chars(fc_name, "% #") : NULL;
 }
 
-/*
+/**
  * Close text window.
  */
 static void text_window_close_cb(Fl_Widget *, void *vtd)
@@ -286,7 +288,7 @@ static void text_window_close_cb(Fl_Widget *, void *vtd)
    delete buf;
 }
 
-/*
+/**
  * Show a new window with the provided text
  */
 void a_Dialog_text_window(const char *title, const char *txt)
@@ -325,14 +327,15 @@ static void choice_cb(Fl_Widget *button, void *number)
 {
   choice_answer = VOIDP2INT(number);
   _MSG("choice_cb: %d\n", choice_answer);
+
   button->window()->hide();
 }
 
-/*
+/**
  * Make a question-dialog with a question and alternatives.
  * Last parameter must be NULL.
  *
- * Return value: 0 = dialog was cancelled, >0 = selected alternative.
+ * @return 0 = dialog was cancelled, >0 = selected alternative.
  */
 int a_Dialog_choice(const char *title, const char *msg, ...)
 {
@@ -354,32 +357,20 @@ int a_Dialog_choice(const char *title, const char *msg, ...)
    int gap = 8;
    int ww = 140 + n * 60, wh = 120;
    int bw = (ww - gap) / n - gap, bh = 45;
-   int ih = 50;
 
    Fl_Window *window = new Fl_Window(ww, wh, title);
    window->set_modal();
    window->begin();
-    Fl_Group *ib = new Fl_Group(0, 0, window->w(), window->h());
-    ib->begin();
-    window->resizable(ib);
- 
-    /* '?' Icon */
-    Fl_Box *o = new Fl_Box(10, (wh - bh - ih) / 2, ih, ih);
-    o->box(FL_THIN_UP_BOX);
-    o->labelfont(FL_TIMES_BOLD);
-    o->labelsize(34);
-    o->color(FL_WHITE);
-    o->labelcolor(FL_BLUE);
-    o->label("?");
-    o->show();
- 
-    if (msg != NULL){
-       Fl_Box *box = new Fl_Box(60, 0, ww - 60, wh - bh, msg);
-       box->labelfont(FL_HELVETICA);
-       box->labelsize(14);
-       box->align(FL_ALIGN_WRAP);
-    }
- 
+
+    Fl_Text_Buffer *buf = new Fl_Text_Buffer();
+    buf->text(msg);
+    Fl_Text_Display *td = new Fl_Text_Display(0, 0, ww, wh - bh);
+    td->buffer(buf);
+    td->textsize((int) rint(14.0 * prefs.font_factor));
+    td->wrap_mode(Fl_Text_Display::WRAP_AT_BOUNDS, 0);
+    
+    window->resizable(td);
+
     int xpos = gap;
     va_start(ap, msg);
     for (i = 1; i <= n; i++) {
@@ -393,10 +384,14 @@ int a_Dialog_choice(const char *title, const char *msg, ...)
     va_end(ap);
    window->end();
 
+   choice_answer = 0;
+
    window->show();
    while (window->shown())
       Fl::wait();
    _MSG("Dialog_choice answer = %d\n", answer);
+   td->buffer(NULL);
+   delete buf;
    delete window;
 
    return choice_answer;
@@ -409,7 +404,7 @@ static void Dialog_user_password_cb(Fl_Widget *button, void *)
    button->window()->hide();
 }
 
-/*
+/**
  * Make a user/password dialog.
  * Call the callback with the result (OK or not) and the given user and
  *   password if OK.
