@@ -23,6 +23,7 @@
 #include "msg.h"
 #include "prefs.h"
 #include "uicmd.hh"
+#include "dialog.hh"
 
 using namespace lout;
 using namespace dw;
@@ -267,7 +268,7 @@ void a_Html_form_display_hiddens2(void *vform, bool display)
  * Form parsing functions
  */
 
-/*
+/**
  * Add an HTML control
  */
 static void Html_add_input(DilloHtml *html, DilloHtmlInputType type,
@@ -290,7 +291,7 @@ static void Html_add_input(DilloHtml *html, DilloHtmlInputType type,
    }
 }
 
-/*
+/**
  * Find radio input by name
  */
 static DilloHtmlInput *Html_get_radio_input(DilloHtml *html, const char *name)
@@ -313,7 +314,7 @@ static DilloHtmlInput *Html_get_radio_input(DilloHtml *html, const char *name)
    return NULL;
 }
 
-/*
+/**
  * Get the current input if available.
  */
 static DilloHtmlInput *Html_get_current_input(DilloHtml *html)
@@ -329,7 +330,7 @@ static DilloHtmlInput *Html_get_current_input(DilloHtml *html)
             inputs->get (inputs->size() - 1) : NULL;
 }
 
-/*
+/**
  * Handle <FORM> tag
  */
 void Html_tag_open_form(DilloHtml *html, const char *tag, int tagsize)
@@ -404,7 +405,7 @@ void Html_tag_close_form(DilloHtml *html)
    html->InFlags &= ~IN_TEXTAREA;
 }
 
-/*
+/**
  * get size, restrict it to reasonable value
  */
 static int Html_input_get_size(DilloHtml *html, const char *attrbuf)
@@ -423,7 +424,7 @@ static int Html_input_get_size(DilloHtml *html, const char *attrbuf)
    return size;
 }
 
-/*
+/**
  * Add a new input to current form
  */
 void Html_tag_open_input(DilloHtml *html, const char *tag, int tagsize)
@@ -579,7 +580,7 @@ void Html_tag_open_input(DilloHtml *html, const char *tag, int tagsize)
    dFree(value);
 }
 
-/*
+/**
  * The ISINDEX tag is just a deprecated form of <INPUT type=text> with
  * implied FORM, afaics.
  */
@@ -627,8 +628,8 @@ void Html_tag_open_textarea(DilloHtml *html, const char *tag, int tagsize)
    html->InFlags |= IN_TEXTAREA;
 }
 
-/*
- * The textarea tag
+/**
+ * The textarea tag.
  */
 void Html_tag_content_textarea(DilloHtml *html, const char *tag, int tagsize)
 {
@@ -658,7 +659,7 @@ void Html_tag_content_textarea(DilloHtml *html, const char *tag, int tagsize)
    } else {
       if (html->DocType != DT_HTML || html->DocTypeVersion <= 4.01f)
          BUG_MSG("<textarea> requires rows attribute.");
-      rows = 10;
+      rows = 3;
    }
    if (rows < 1 || rows > MAX_ROWS) {
       int badRows = rows;
@@ -685,8 +686,8 @@ void Html_tag_content_textarea(DilloHtml *html, const char *tag, int tagsize)
    dFree(name);
 }
 
-/*
- * Close  textarea
+/**
+ * Close textarea.
  * (TEXTAREA is parsed in VERBATIM mode, and entities are handled here)
  */
 void Html_tag_close_textarea(DilloHtml *html)
@@ -943,7 +944,7 @@ void Html_tag_open_button(DilloHtml *html, const char *tag, int tagsize)
       /* We used to have Textblock (prefs.limit_text_width, ...) here,
        * but it caused 100% CPU usage.
        */
-      page = new Textblock (false);
+      page = new Textblock (false, true);
       page->setStyle (html->backgroundStyle ());
 
       ResourceFactory *factory = HT2LT(html)->getResourceFactory();
@@ -951,9 +952,7 @@ void Html_tag_open_button(DilloHtml *html, const char *tag, int tagsize)
       embed = new Embed(resource);
 // a_Dw_button_set_sensitive (DW_BUTTON (button), FALSE);
 
-      HT2TB(html)->addParbreak (5, html->wordStyle ());
       HT2TB(html)->addWidget (embed, html->backgroundStyle ());
-      HT2TB(html)->addParbreak (5, html->wordStyle ());
 
       S_TOP(html)->textblock = html->dw = page;
 
@@ -967,7 +966,7 @@ void Html_tag_open_button(DilloHtml *html, const char *tag, int tagsize)
    dFree(type);
 }
 
-/*
+/**
  * Handle close <BUTTON>
  */
 void Html_tag_close_button(DilloHtml *html)
@@ -1039,6 +1038,16 @@ void DilloHtmlForm::eventHandler(Resource *resource, EventButton *event)
  */
 void DilloHtmlForm::submit(DilloHtmlInput *active_input, EventButton *event)
 {
+   if (!dStrAsciiCasecmp(URL_SCHEME(html->page_url), "https") &&
+       dStrAsciiCasecmp(URL_SCHEME(action), "https")) {
+      int choice = a_Dialog_choice("Dillo: Insecure form submission",
+                                   "A form on a SECURE page wants to use an "
+                                   "INSECURE protocol to submit data.",
+                                   "Continue", "Cancel", NULL);
+      if (choice != 1)
+         return;
+   }
+
    DilloUrl *url = buildQueryUrl(active_input);
    if (url) {
       if (event && event->button == 2) {
@@ -1056,7 +1065,7 @@ void DilloHtmlForm::submit(DilloHtmlInput *active_input, EventButton *event)
    }
 }
 
-/*
+/**
  * Build a new query URL.
  * (Called by submit())
  */
@@ -1114,7 +1123,7 @@ DilloUrl *DilloHtmlForm::buildQueryUrl(DilloHtmlInput *active_input)
    return new_url;
 }
 
-/*
+/**
  * Construct the data for a query URL
  */
 Dstr *DilloHtmlForm::buildQueryData(DilloHtmlInput *active_submit)
@@ -1236,7 +1245,7 @@ Dstr *DilloHtmlForm::buildQueryData(DilloHtmlInput *active_submit)
    return DataStr;
 }
 
-/*
+/**
  * Generate a boundary string for use in separating the parts of a
  * multipart/form-data submission.
  */
@@ -1299,7 +1308,7 @@ char *DilloHtmlForm::makeMultipartBoundary(iconv_t char_encoder,
    return ret;
 }
 
-/*
+/**
  * Pass input text through character set encoder.
  * Return value: same input Dstr if no encoding is needed.
  *               new Dstr when encoding (input Dstr is freed).
@@ -1368,7 +1377,7 @@ Dstr *DilloHtmlForm::encodeText(iconv_t char_encoder, Dstr **input)
    return output;
 }
 
-/*
+/**
  * Urlencode 'str' and append it to 'dstr'
  */
 void DilloHtmlForm::strUrlencodeAppend(Dstr *dstr, const char *str)
@@ -1378,7 +1387,7 @@ void DilloHtmlForm::strUrlencodeAppend(Dstr *dstr, const char *str)
    dFree(encoded);
 }
 
-/*
+/**
  * Append a name-value pair to url data using url encoding.
  */
 void DilloHtmlForm::inputUrlencodeAppend(Dstr *data, const char *name,
@@ -1392,7 +1401,7 @@ void DilloHtmlForm::inputUrlencodeAppend(Dstr *data, const char *name,
    }
 }
 
-/*
+/**
  * Append files to URL data using multipart encoding.
  * Currently only accepts one file.
  */
@@ -1445,7 +1454,7 @@ void DilloHtmlForm::filesInputMultipartAppend(Dstr* data,
    }
 }
 
-/*
+/**
  * Append a name-value pair to url data using multipart encoding.
  */
 void DilloHtmlForm::inputMultipartAppend(Dstr *data,
@@ -1468,7 +1477,7 @@ void DilloHtmlForm::inputMultipartAppend(Dstr *data,
    }
 }
 
-/*
+/**
  * Append an image button click position to url data using url encoding.
  */
 void DilloHtmlForm::imageInputUrlencodeAppend(Dstr *data, Dstr *name, Dstr *x,
@@ -1483,7 +1492,7 @@ void DilloHtmlForm::imageInputUrlencodeAppend(Dstr *data, Dstr *name, Dstr *x,
       dStr_sprintfa(data, "x=%s&y=%s&", x->str, y->str);
 }
 
-/*
+/**
  * Append an image button click position to url data using multipart encoding.
  */
 void DilloHtmlForm::imageInputMultipartAppend(Dstr *data, const char *boundary,
@@ -1502,7 +1511,7 @@ void DilloHtmlForm::imageInputMultipartAppend(Dstr *data, const char *boundary,
    dStr_truncate(name, orig_len);
 }
 
-/*
+/**
  * Reset all inputs containing reset to their initial values.  In
  * general, reset is the reset button for the form.
  */
@@ -1513,7 +1522,7 @@ void DilloHtmlForm::reset ()
       inputs->get(i)->reset();
 }
 
-/*
+/**
  * Show/hide "hidden" form controls
  */
 void DilloHtmlForm::display_hiddens(bool display)
@@ -1534,7 +1543,7 @@ void DilloHtmlForm::setEnabled(bool enabled)
       inputs->get(i)->setEnabled(enabled);
 }
 
-/*
+/**
  * Add a new input.
  */
 void DilloHtmlForm::addInput(DilloHtmlInput *input, DilloHtmlInputType type)
@@ -1552,7 +1561,7 @@ void DilloHtmlForm::addInput(DilloHtmlInput *input, DilloHtmlInputType type)
    }
 }
 
-/*
+/**
  * Return the input with a given resource.
  */
 DilloHtmlInput *DilloHtmlForm::getInput (Resource *resource)
@@ -1566,7 +1575,7 @@ DilloHtmlInput *DilloHtmlForm::getInput (Resource *resource)
    return NULL;
 }
 
-/*
+/**
  * Return a Radio input for the given name.
  */
 DilloHtmlInput *DilloHtmlForm::getRadioInput (const char *name)
@@ -1592,7 +1601,7 @@ void DilloHtmlReceiver::activate (Resource *resource)
    form->eventHandler(resource, NULL);
 }
 
-/*
+/**
  * Enter a form control, as in "onmouseover".
  * For _pressing_ enter in a text control, see activate().
  */
@@ -1615,7 +1624,7 @@ void DilloHtmlReceiver::enter (Resource *resource)
    a_UIcmd_set_msg(html->bw, "%s", msg);
 }
 
-/*
+/**
  * Leave a form control, or "onmouseout".
  */
 void DilloHtmlReceiver::leave (Resource *resource)
@@ -1671,7 +1680,7 @@ DilloHtmlInput::~DilloHtmlInput ()
       delete select;
 }
 
-/*
+/**
  * Connect to a receiver.
  */
 void DilloHtmlInput::connectTo(DilloHtmlReceiver *form_receiver)
@@ -1693,7 +1702,7 @@ void DilloHtmlInput::connectTo(DilloHtmlReceiver *form_receiver)
    }
 }
 
-/*
+/**
  * Activate a form
  */
 void DilloHtmlInput::activate(DilloHtmlForm *form, int num_entry_fields,
@@ -1725,7 +1734,7 @@ void DilloHtmlInput::activate(DilloHtmlForm *form, int num_entry_fields,
    }
 }
 
-/*
+/**
  * Read a file into cache
  */
 void DilloHtmlInput::readFile (BrowserWindow *bw)
@@ -1745,7 +1754,7 @@ void DilloHtmlInput::readFile (BrowserWindow *bw)
    }
 }
 
-/*
+/**
  * Get the values for a "successful control".
  */
 void DilloHtmlInput::appendValuesTo(Dlist *values, bool is_active_submit)
@@ -1821,7 +1830,7 @@ void DilloHtmlInput::appendValuesTo(Dlist *values, bool is_active_submit)
    }
 }
 
-/*
+/**
  * Reset to the initial value.
  */
 void DilloHtmlInput::reset ()
@@ -1904,7 +1913,7 @@ void DilloHtmlSelect::addOpt (DilloHtmlOptbase *opt)
    opts->set (size, opt);
 }
 
-/*
+/**
  * Select the first option if nothing else is selected.
  */
 void DilloHtmlSelect::ensureSelection()
@@ -1998,7 +2007,7 @@ DilloHtmlOption::~DilloHtmlOption ()
  * Utilities
  */
 
-/*
+/**
  * Create input image for the form
  */
 static Embed *Html_input_image(DilloHtml *html, const char *tag, int tagsize)
