@@ -2,6 +2,7 @@
  * File: menu.cc
  *
  * Copyright (C) 2005-2007 Jorge Arellano Cid <jcid@dillo.org>
+ * Copyright (C) 2024 Rodrigo Arias Mallo <rodarima@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -9,7 +10,10 @@
  * (at your option) any later version.
  */
 
-// Functions/Methods for menus
+/**
+ * @file
+ * Functions for menus
+ */
 
 #include <FL/Fl.H>
 #include <FL/Fl_Menu_Item.H>
@@ -57,7 +61,7 @@ static void Menu_nop_cb(Fl_Widget*, void*)
 {
 }
 
-/*
+/**
  * Static function for File menu callbacks.
  */
 static void filemenu_cb(Fl_Widget*, void *data)
@@ -86,7 +90,7 @@ static void Menu_copy_urlstr_cb(Fl_Widget*, void *user_data)
    }
 }
 
-/*
+/**
  * Open URL
  */
 static void Menu_open_url_cb(Fl_Widget*, void *user_data)
@@ -96,7 +100,7 @@ static void Menu_open_url_cb(Fl_Widget*, void *user_data)
    a_UIcmd_open_url(popup_bw, url);
 }
 
-/*
+/**
  * Open URL in new window
  */
 static void Menu_open_url_nw_cb(Fl_Widget*, void *user_data)
@@ -106,7 +110,7 @@ static void Menu_open_url_nw_cb(Fl_Widget*, void *user_data)
    a_UIcmd_open_url_nw(popup_bw, url);
 }
 
-/*
+/**
  * Open URL in new Tab
  */
 static void Menu_open_url_nt_cb(Fl_Widget*, void *user_data)
@@ -117,7 +121,7 @@ static void Menu_open_url_nt_cb(Fl_Widget*, void *user_data)
    a_UIcmd_open_url_nt(popup_bw, url, focus);
 }
 
-/*
+/**
  * Add bookmark
  */
 static void Menu_add_bookmark_cb(Fl_Widget*, void *user_data)
@@ -126,7 +130,7 @@ static void Menu_add_bookmark_cb(Fl_Widget*, void *user_data)
    a_UIcmd_add_bookmark(popup_bw, url);
 }
 
-/*
+/**
  * Find text
  */
 static void Menu_find_text_cb(Fl_Widget*, void*)
@@ -134,7 +138,7 @@ static void Menu_find_text_cb(Fl_Widget*, void*)
    ((UI *)popup_bw->ui)->findbar_toggle(1);
 }
 
-/*
+/**
  * Save link
  */
 static void Menu_save_link_cb(Fl_Widget*, void *user_data)
@@ -143,7 +147,7 @@ static void Menu_save_link_cb(Fl_Widget*, void *user_data)
    a_UIcmd_save_link(popup_bw, url);
 }
 
-/*
+/**
  * Save current page
  */
 static void Menu_save_page_cb(Fl_Widget*, void*)
@@ -151,7 +155,7 @@ static void Menu_save_page_cb(Fl_Widget*, void*)
    a_UIcmd_save(popup_bw);
 }
 
-/*
+/**
  * View current page source
  */
 static void Menu_view_page_source_cb(Fl_Widget*, void *user_data)
@@ -160,7 +164,7 @@ static void Menu_view_page_source_cb(Fl_Widget*, void *user_data)
    a_UIcmd_view_page_source(popup_bw, url);
 }
 
-/*
+/**
  * View current page's bugs
  */
 static void Menu_view_page_bugs_cb(Fl_Widget*, void*)
@@ -168,7 +172,7 @@ static void Menu_view_page_bugs_cb(Fl_Widget*, void*)
    a_UIcmd_view_page_bugs(popup_bw);
 }
 
-/*
+/**
  * Load images on current page that match URL pattern
  */
 static void Menu_load_images_cb(Fl_Widget*, void *user_data)
@@ -180,7 +184,7 @@ static void Menu_load_images_cb(Fl_Widget*, void *user_data)
       a_Html_load_images(doc, popup_url);
 }
 
-/*
+/**
  * Submit form
  */
 static void Menu_form_submit_cb(Fl_Widget*, void*)
@@ -191,7 +195,7 @@ static void Menu_form_submit_cb(Fl_Widget*, void*)
       a_Html_form_submit(doc, popup_form);
 }
 
-/*
+/**
  * Reset form
  */
 static void Menu_form_reset_cb(Fl_Widget*, void*)
@@ -202,7 +206,7 @@ static void Menu_form_reset_cb(Fl_Widget*, void*)
       a_Html_form_reset(doc, popup_form);
 }
 
-/*
+/**
  * Toggle display of 'hidden' form controls.
  */
 static void Menu_form_hiddens_cb(Fl_Widget*, void *user_data)
@@ -232,42 +236,58 @@ static void Menu_stylesheet_cb(Fl_Widget*, void *vUrl)
    }
 }
 
-/*
- * Validate URL with the W3C
+static void Menu_bugmeter_validate(const char *validator_url)
+{
+   if (popup_url &&
+       dStrAsciiCasecmp(URL_SCHEME(popup_url), "dpi")) {
+      const char *popup_str = URL_STR(popup_url),
+                 *ptr = strrchr(popup_str, '#');
+      char *no_fragment = ptr ? dStrndup(popup_str, ptr - popup_str)
+                              : dStrdup(popup_str);
+      char *encoded = a_Url_encode_hex_str(no_fragment);
+      Dstr *dstr = dStr_sized_new(128);
+
+      dStr_sprintf(dstr, validator_url, encoded);
+      a_UIcmd_open_urlstr(popup_bw, dstr->str);
+      dStr_free(dstr, 1);
+      dFree(encoded);
+      dFree(no_fragment);
+   }
+}
+
+/**
+ * Validate URL with the W3C Nu validator (for HTML 5)
+ */
+static void Menu_bugmeter_validate_w3c_nu_cb(Fl_Widget*, void*)
+{
+   Menu_bugmeter_validate(
+         "https://validator.w3.org/nu/"
+         "?useragent=Validator.nu%%2FLV+https%%3A%%2F%%2Fvalidator.w3.org%%2Fservices"
+         "&acceptlanguage="
+         "&doc=%s");
+}
+
+/**
+ * Validate URL with the W3C legacy validator (HTML 4.01 and older)
  */
 static void Menu_bugmeter_validate_w3c_cb(Fl_Widget*, void*)
 {
-   Dstr *dstr = dStr_sized_new(128);
-
-   dStr_sprintf(dstr, "http://validator.w3.org/check?uri=%s",
-                URL_STR(popup_url));
-   a_UIcmd_open_urlstr(popup_bw, dstr->str);
-   dStr_free(dstr, 1);
+   Menu_bugmeter_validate(
+         "https://validator.w3.org/check?uri=%s"
+         "&charset=%%28detect+automatically%%29"
+         "&doctype=Inline&group=0"
+         "&user-agent=W3C_Validator%%2F1.3+");
 }
 
-/*
- * Validate URL with the WDG
- */
-static void Menu_bugmeter_validate_wdg_cb(Fl_Widget*, void*)
-{
-   Dstr *dstr = dStr_sized_new(128);
-
-   dStr_sprintf(dstr,
-      "http://www.htmlhelp.org/cgi-bin/validate.cgi?url=%s&warnings=yes",
-      URL_STR(popup_url));
-   a_UIcmd_open_urlstr(popup_bw, dstr->str);
-   dStr_free(dstr, 1);
-}
-
-/*
+/**
  * Show info page for the bug meter
  */
 static void Menu_bugmeter_about_cb(Fl_Widget*, void*)
 {
-   a_UIcmd_open_urlstr(popup_bw, "http://www.dillo.org/help/bug_meter.html");
+   a_UIcmd_open_urlstr(popup_bw, "https://dillo-browser.github.io/old/help/bug_meter.html");
 }
 
-/*
+/**
  * Navigation History callback.
  * Go to selected URL.
  */
@@ -326,7 +346,7 @@ static void Menu_popup_cb(void *data)
    a_Timeout_remove();
 }
 
-/*
+/**
  * Page popup menu (construction & popup)
  */
 void a_Menu_page_popup(BrowserWindow *bw, const DilloUrl *url,
@@ -428,7 +448,7 @@ static void Menu_set_link_menu_user_data(void *user_data)
       link_menu[i].user_data(user_data);
 }
 
-/*
+/**
  * Link popup menu (construction & popup)
  */
 void a_Menu_link_popup(BrowserWindow *bw, const DilloUrl *url)
@@ -446,7 +466,7 @@ void a_Menu_link_popup(BrowserWindow *bw, const DilloUrl *url)
    a_Timeout_add(0.0, Menu_popup_cb, (void*)&link_data);
 }
 
-/*
+/**
  * Image popup menu (construction & popup)
  */
 void a_Menu_image_popup(BrowserWindow *bw, const DilloUrl *url,
@@ -505,7 +525,7 @@ void a_Menu_image_popup(BrowserWindow *bw, const DilloUrl *url,
    a_Timeout_add(0.0, Menu_popup_cb, (void*)&image_data);
 }
 
-/*
+/**
  * Form popup menu (construction & popup)
  */
 void a_Menu_form_popup(BrowserWindow *bw, const DilloUrl *page_url,
@@ -533,7 +553,7 @@ void a_Menu_form_popup(BrowserWindow *bw, const DilloUrl *page_url,
    a_Timeout_add(0.0, Menu_popup_cb, (void*)&form_data);
 }
 
-/*
+/**
  * File popup menu (construction & popup)
  */
 void a_Menu_file_popup(BrowserWindow *bw, void *v_wid)
@@ -566,16 +586,18 @@ void a_Menu_file_popup(BrowserWindow *bw, void *v_wid)
    a_Timeout_add(0.0, Menu_simple_popup_cb, (void*)pm);
 }
 
-/*
+/**
  * Bugmeter popup menu (construction & popup)
  */
 void a_Menu_bugmeter_popup(BrowserWindow *bw, const DilloUrl *url)
 {
    static Fl_Menu_Item pm[] = {
-      {"Validate URL with W3C", 0, Menu_bugmeter_validate_w3c_cb,0,0,0,0,0,0},
-      {"Validate URL with WDG", 0, Menu_bugmeter_validate_wdg_cb, 0,
-       FL_MENU_DIVIDER,0,0,0,0},
-      {"About bug meter", 0, Menu_bugmeter_about_cb,0,0,0,0,0,0},
+      {"Validate URL with W3C Nu validator (HTML5 only)", 0,
+         Menu_bugmeter_validate_w3c_nu_cb,0,0,0,0,0,0},
+      {"Validate URL with W3C validator (HTML 4.01 and older)", 0,
+         Menu_bugmeter_validate_w3c_cb,0,FL_MENU_DIVIDER,0,0,0,0},
+      {"About bug meter", 0,
+         Menu_bugmeter_about_cb,0,0,0,0,0,0},
       {0,0,0,0,0,0,0,0,0}
    };
 
@@ -588,8 +610,8 @@ void a_Menu_bugmeter_popup(BrowserWindow *bw, const DilloUrl *url)
    a_Timeout_add(0.0, Menu_simple_popup_cb, (void*)pm);
 }
 
-/*
- * Navigation History popup menu (construction & popup)
+/**
+ * Navigation History popup menu (construction & popup).
  *
  * direction: {backward = -1, forward = 1}
  */
@@ -625,7 +647,7 @@ void a_Menu_history_popup(BrowserWindow *bw, int x, int y, int direction)
    a_Timeout_add(0.0, Menu_simple_popup_cb, (void*)pm);
 }
 
-/*
+/**
  * Toggle use of remote stylesheets
  */
 static void Menu_remote_css_cb(Fl_Widget *wid, void*)
@@ -637,7 +659,7 @@ static void Menu_remote_css_cb(Fl_Widget *wid, void*)
    a_UIcmd_repush(popup_bw);
 }
 
-/*
+/**
  * Toggle use of embedded CSS style
  */
 static void Menu_embedded_css_cb(Fl_Widget *wid, void*)
@@ -649,6 +671,19 @@ static void Menu_embedded_css_cb(Fl_Widget *wid, void*)
    a_UIcmd_repush(popup_bw);
 }
 
+
+/**
+ * Toggle use of force https mode
+ */
+static void Menu_force_https_cb(Fl_Widget *wid, void*)
+{
+   Fl_Menu_Item *item = (Fl_Menu_Item*) wid;
+
+   item->flags ^= FL_MENU_VALUE;
+   prefs.http_force_https = item->flags & FL_MENU_VALUE ? 1 : 0;
+   a_UIcmd_repush(popup_bw);
+}
+
 static void Menu_panel_change_cb(Fl_Widget*, void *user_data)
 {
    UI *ui = (UI*)popup_bw->ui;
@@ -659,7 +694,7 @@ static void Menu_panel_change_cb(Fl_Widget*, void *user_data)
       ui->change_panel(VOIDP2INT(user_data), ui->get_smallicons());
 }
 
-/*
+/**
  * Toggle loading of images -- and load them if enabling.
  */
 static void Menu_imgload_toggle_cb(Fl_Widget *wid, void*)
@@ -678,7 +713,7 @@ static void Menu_imgload_toggle_cb(Fl_Widget *wid, void*)
    }
 }
 
-/*
+/**
  * Toggle loading of background images.
  */
 static void Menu_bgimg_load_toggle_cb(Fl_Widget *wid, void*)
@@ -690,8 +725,8 @@ static void Menu_bgimg_load_toggle_cb(Fl_Widget *wid, void*)
    a_UIcmd_repush(popup_bw);
 }
 
-/*
- * Tools popup menu (construction & popup)
+/**
+ * Tools popup menu (construction & popup).
  */
 void a_Menu_tools_popup(BrowserWindow *bw, int x, int y)
 {
@@ -706,6 +741,8 @@ void a_Menu_tools_popup(BrowserWindow *bw, int x, int y)
        FL_MENU_TOGGLE,0,0,0,0},
       {"Load background images", 0, Menu_bgimg_load_toggle_cb, 0,
        FL_MENU_TOGGLE|FL_MENU_DIVIDER,0,0,0,0},
+      {"Force HTTPS", 0, Menu_force_https_cb, 0,
+       FL_MENU_TOGGLE|FL_MENU_DIVIDER,0,0,0,0},
       {"Panel size", 0, Menu_nop_cb, (void*)"Submenu1", FL_SUBMENU,0,0,0,0},
          {"tiny",  0,Menu_panel_change_cb,(void*)0,FL_MENU_RADIO,0,0,0,0},
          {"small", 0,Menu_panel_change_cb,(void*)1,FL_MENU_RADIO,0,0,0,0},
@@ -729,8 +766,10 @@ void a_Menu_tools_popup(BrowserWindow *bw, int x, int y)
       pm[2].set();
    if (prefs.load_background_images)
       pm[3].set();
-   pm[5+cur_panelsize].setonly();
-   cur_smallicons ? pm[8].set() : pm[8].clear();
+   if (prefs.http_force_https)
+      pm[4].set();
+   pm[6+cur_panelsize].setonly();
+   cur_smallicons ? pm[9].set() : pm[9].clear();
 
    item = pm->popup(x, y);
    if (item) {
