@@ -9,7 +9,8 @@
  * (at your option) any later version.
  */
 
-/* Support for a navigation stack */
+/** @file
+ * Support for a navigation stack */
 
 #include <stdio.h>
 #include <sys/stat.h>
@@ -353,6 +354,7 @@ void a_Nav_push(BrowserWindow *bw, const DilloUrl *url,
    a_Nav_cancel_expect(bw);
    a_Bw_expect(bw, url);
    Nav_open_url(bw, url, requester, 0);
+   a_UIcmd_set_location_text(bw, URL_STR(url));
 }
 
 /*
