@@ -2,6 +2,7 @@
  * Preferences
  *
  * Copyright (C) 2006-2009 Jorge Arellano Cid <jcid@dillo.org>
+ * Copyright (C) 2024 Rodrigo Arias Mallo <rodarima@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,13 +13,13 @@
 #include "prefs.h"
 
 #define PREFS_START_PAGE      "about:splash"
-#define PREFS_HOME            "http://www.dillo.org/"
+#define PREFS_HOME            "https://dillo-browser.github.io/"
 #define PREFS_FONT_SERIF      "DejaVu Serif"
 #define PREFS_FONT_SANS_SERIF "DejaVu Sans"
-#define PREFS_FONT_CURSIVE    "URW Chancery L"
+#define PREFS_FONT_CURSIVE    "DejaVu Sans"
 #define PREFS_FONT_FANTASY    "DejaVu Sans" /* TODO: find good default */
 #define PREFS_FONT_MONOSPACE  "DejaVu Sans Mono"
-#define PREFS_SEARCH_URL      "dd http://duckduckgo.com/lite/?kp=-1&q=%s"
+#define PREFS_SEARCH_URL      "dd http://duckduckgo.com/lite/?kp=-1&kd=-1&q=%s"
 #define PREFS_NO_PROXY        "localhost 127.0.0.1"
 #define PREFS_SAVE_DIR        "/tmp/"
 #define PREFS_HTTP_REFERER    "host"
@@ -30,7 +31,7 @@
  *---------------------------------------------------------------------------*/
 DilloPrefs prefs;
 
-/*
+/**
  * Sets the default settings.
  */
 
@@ -63,20 +64,28 @@ void a_Prefs_init(void)
    prefs.http_language = NULL;
    prefs.http_proxy = NULL;
    prefs.http_max_conns = 6;
+   prefs.http_persistent_conns = TRUE;
    prefs.http_proxyuser = NULL;
    prefs.http_referer = dStrdup(PREFS_HTTP_REFERER);
+   prefs.http_strict_transport_security = TRUE;
+   prefs.http_force_https = FALSE;
    prefs.http_user_agent = dStrdup(PREFS_HTTP_USER_AGENT);
    prefs.limit_text_width = FALSE;
+   prefs.adjust_min_width = TRUE;
+   prefs.adjust_table_min_width = TRUE;
    prefs.load_images=TRUE;
    prefs.load_background_images=FALSE;
    prefs.load_stylesheets=TRUE;
    prefs.middle_click_drags_page = TRUE;
    prefs.middle_click_opens_new_tab = TRUE;
-   prefs.right_click_closes_tab = FALSE;
+   prefs.right_click_closes_tab = TRUE;
+   prefs.scroll_switches_tabs = TRUE;
+   prefs.scroll_switches_tabs_reverse = FALSE;
    prefs.no_proxy = dStrdup(PREFS_NO_PROXY);
    prefs.panel_size = P_medium;
    prefs.parse_embedded_css=TRUE;
    prefs.save_dir = dStrdup(PREFS_SAVE_DIR);
+   prefs.scroll_step = 100;
    prefs.search_urls = dList_new(16);
    dList_append(prefs.search_urls, dStrdup(PREFS_SEARCH_URL));
    prefs.search_url_idx = 0;
@@ -109,9 +118,9 @@ void a_Prefs_init(void)
    prefs.ui_tab_active_bg_color = -1;
    prefs.ui_tab_bg_color = -1;
    prefs.ui_tab_active_fg_color = -1;
+   prefs.ui_tab_height = 20;
    prefs.ui_tab_fg_color = -1;
    prefs.ui_text_bg_color = -1;
-   prefs.w3c_plus_heuristics = TRUE;
 
    prefs.penalty_hyphen = 100;
    prefs.penalty_hyphen_2 = 800;
@@ -121,8 +130,8 @@ void a_Prefs_init(void)
    prefs.stretchability_factor = 100;
 }
 
-/*
- *  memory-deallocation
+/**
+ *  memory-deallocation.
  *  (Call this one at exit time)
  */
 void a_Prefs_freeall(void)
