@@ -2,6 +2,7 @@
  * Preferences
  *
  * Copyright (C) 2006-2009 Jorge Arellano Cid <jcid@dillo.org>
+ * Copyright (C) 2024 Rodrigo Arias Mallo <rodarima@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,7 +31,7 @@ extern "C" {
 #define PREFS_UI_TAB_BG_COLOR 19
 #define PREFS_UI_TAB_FG_COLOR 20
 
-/* Panel sizes */
+/** Panel sizes. */
 enum { P_tiny = 0, P_small, P_medium };
 
 typedef struct {
@@ -58,6 +59,7 @@ typedef struct {
    int32_t ui_tab_active_fg_color;
    int32_t ui_tab_bg_color;
    int32_t ui_tab_fg_color;
+   int32_t ui_tab_height;
    int32_t ui_text_bg_color;
    bool_t contrast_visited_color;
    bool_t show_tooltip;
@@ -66,11 +68,13 @@ typedef struct {
    int panel_size;
    bool_t small_icons;
    bool_t limit_text_width;
-   bool_t w3c_plus_heuristics;
+   bool_t adjust_min_width;
+   bool_t adjust_table_min_width;
    bool_t focus_new_tab;
    double font_factor;
    int32_t font_max_size;
    int32_t font_min_size;
+   int32_t scroll_step;
    bool_t show_back;
    bool_t show_forw;
    bool_t show_home;
@@ -91,6 +95,9 @@ typedef struct {
    bool_t load_background_images;
    bool_t load_stylesheets;
    bool_t parse_embedded_css;
+   bool_t http_persistent_conns;
+   bool_t http_strict_transport_security;
+   bool_t http_force_https;
    int32_t buffered_drawing;
    char *font_serif;
    char *font_sans_serif;
@@ -100,6 +107,8 @@ typedef struct {
    bool_t enterpress_forces_submit;
    bool_t middle_click_opens_new_tab;
    bool_t right_click_closes_tab;
+   bool_t scroll_switches_tabs;
+   bool_t scroll_switches_tabs_reverse;
    bool_t search_url_idx;
    Dlist *search_urls;
    char *save_dir;
@@ -111,7 +120,7 @@ typedef struct {
    int stretchability_factor;
 } DilloPrefs;
 
-/* Global Data */
+/** Global Data */
 extern DilloPrefs prefs;
 
 void a_Prefs_init(void);
