@@ -2,6 +2,7 @@
  * File: ui.cc
  *
  * Copyright (C) 2005-2007 Jorge Arellano Cid <jcid@dillo.org>
+ * Copyright (C) 2024 Rodrigo Arias Mallo <rodarima@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -9,10 +10,13 @@
  * (at your option) any later version.
  */
 
-// UI for Dillo
+/** @file
+ * UI for Dillo
+ */
 
 #include <unistd.h>
 #include <stdio.h>
+#include <math.h>      /* rint */
 
 #include "keys.hh"
 #include "ui.hh"
@@ -86,8 +90,8 @@ static struct iconset *icons = &standard_icons;
 
 //----------------------------------------------------------------------------
 
-/*
- * (Used to avoid certain shortcuts in the location bar)
+/**
+ * Used to avoid certain shortcuts in the location bar
  */
 class CustInput : public TipWinInput {
 public:
@@ -96,7 +100,7 @@ public:
    virtual int handle(int e);
 };
 
-/*
+/**
  * Disable keys: Up, Down, Page_Up, Page_Down, Tab and
  * CTRL+{o,r,Home,End}  SHIFT+{Left,Right}.
  */
@@ -161,7 +165,7 @@ int CustInput::handle(int e)
 
 //----------------------------------------------------------------------------
 
-/*
+/**
  * Used to handle "paste" within the toolbar's Clear button.
  */
 class CustPasteButton : public CustButton {
@@ -186,7 +190,7 @@ int CustPasteButton::handle(int e)
 
 //----------------------------------------------------------------------------
 
-/*
+/**
  * Used to resize the progress boxes automatically.
  */
 class CustProgressBox : public Fl_Box {
@@ -218,7 +222,7 @@ public:
 // Callback functions --------------------------------------------------------
 //
 
-/*
+/**
  * Callback for the search button.
  */
 static void search_cb(Fl_Widget *wid, void *data)
@@ -230,7 +234,7 @@ static void search_cb(Fl_Widget *wid, void *data)
    }
 }
 
-/*
+/**
  * Callback for the help button.
  */
 static void help_cb(Fl_Widget *w, void *)
@@ -245,12 +249,12 @@ static void help_cb(Fl_Widget *w, void *)
    } else {
       MSG("Can't read local help file at \"%s\"."
           " Getting remote help...\n", path);
-      a_UIcmd_open_urlstr(bw, "http://www.dillo.org/dillo3-help.html");
+      a_UIcmd_open_urlstr(bw, "https://dillo-browser.github.io/user_help.html");
    }
    dFree(path);
 }
 
-/*
+/**
  * Callback for the File menu button.
  */
 static void filemenu_cb(Fl_Widget *wid, void *)
@@ -261,7 +265,7 @@ static void filemenu_cb(Fl_Widget *wid, void *)
    }
 }
 
-/*
+/**
  * Callback for the location's clear-button.
  */
 static void clear_cb(Fl_Widget *w, void *data)
@@ -272,12 +276,12 @@ static void clear_cb(Fl_Widget *w, void *data)
    if (b == FL_LEFT_MOUSE) {
       ui->set_location("");
       ui->focus_location();
-   } if (b == FL_MIDDLE_MOUSE) {
+   } else if (b == FL_MIDDLE_MOUSE) {
       ui->paste_url();
    }
 }
 
-/*
+/**
  * Send the browser to the new URL in the location.
  */
 static void location_cb(Fl_Widget *wid, void *data)
@@ -293,7 +297,7 @@ static void location_cb(Fl_Widget *wid, void *data)
 }
 
 
-/*
+/**
  * Callback handler for button press on the panel
  */
 static void b1_cb(Fl_Widget *wid, void *cb_data)
@@ -357,7 +361,7 @@ static void b1_cb(Fl_Widget *wid, void *cb_data)
    }
 }
 
-/*
+/**
  * Callback for the bug meter button.
  */
 static void bugmeter_cb(Fl_Widget *wid, void *data)
@@ -378,7 +382,7 @@ static void bugmeter_cb(Fl_Widget *wid, void *data)
 // Panel construction methods
 //----------------------------
 
-/*
+/**
  * Make a generic navigation button
  */
 CustButton *UI::make_button(const char *label, Fl_Image *img, Fl_Image *deimg,
@@ -401,7 +405,7 @@ CustButton *UI::make_button(const char *label, Fl_Image *img, Fl_Image *deimg,
    return b;
 }
 
-/*
+/**
  * Create the archetipic browser buttons
  */
 void UI::make_toolbar(int tw, int th)
@@ -440,7 +444,7 @@ void UI::make_toolbar(int tw, int th)
    Tools->set_tooltip("Settings");
 }
 
-/*
+/**
  * Create the location box (Clear/Input/Search)
  */
 void UI::make_location(int ww)
@@ -462,6 +466,7 @@ void UI::make_location(int ww)
      i->when(FL_WHEN_ENTER_KEY);
      i->callback(location_cb, this);
      i->set_tooltip("Location");
+     i->textsize((int) rint(14.0 * prefs.font_factor));
      p_xpos += i->w();
     LocationGroup->box(FL_THIN_UP_BOX);   // or FL_FLAT_BOX
     LocationGroup->end();
@@ -484,7 +489,7 @@ void UI::make_location(int ww)
 
 }
 
-/*
+/**
  * Create the progress bars
  */
 void UI::make_progress_bars(int wide, int thin_up)
@@ -502,11 +507,11 @@ void UI::make_progress_bars(int wide, int thin_up)
     PProg->update_label(wide ? "Page\n0.0 KB" : "0.0 KB");
 }
 
-/*
- * Create the "File" menu
+/**
+ * Create the "File" menu.
  * Static function for File menu callbacks.
  */
-Fl_Widget *UI::make_filemenu_button()
+void UI::make_filemenu_button()
 {
    CustButton *btn;
    int w = 0, h = 0, padding;
@@ -527,11 +532,10 @@ Fl_Widget *UI::make_filemenu_button()
    btn->clear_visible_focus();
    if (!prefs.show_filemenu)
       btn->hide();
-   return btn;
 }
 
 
-/*
+/**
  * Create the control panel
  */
 void UI::make_panel(int ww)
@@ -611,7 +615,7 @@ void UI::make_panel(int ww)
    }
 }
 
-/*
+/**
  * Create the status panel
  */
 void UI::make_status_bar(int ww, int wh)
@@ -622,7 +626,7 @@ void UI::make_status_bar(int ww, int wh)
 
     // Status box
     StatusOutput = new Fl_Output(0, wh-sh, ww-bm_w, sh);
-    StatusOutput->value("http://www.dillo.org");
+    StatusOutput->value("https://dillo-browser.github.io/");
     StatusOutput->labelsize(8);
     StatusOutput->box(FL_THIN_DOWN_BOX);
     StatusOutput->clear_visible_focus();
@@ -642,7 +646,7 @@ void UI::make_status_bar(int ww, int wh)
    StatusBar->rearrange();
 }
 
-/*
+/**
  * User Interface constructor
  */
 UI::UI(int x, int y, int ui_w, int ui_h, const char* label, const UI *cur_ui) :
@@ -699,7 +703,7 @@ UI::UI(int x, int y, int ui_w, int ui_h, const char* label, const UI *cur_ui) :
    }
 }
 
-/*
+/**
  * UI destructor
  */
 UI::~UI()
@@ -707,7 +711,7 @@ UI::~UI()
    _MSG("UI::~UI()\n");
 }
 
-/*
+/**
  * FLTK event handler for this window.
  */
 int UI::handle(int event)
@@ -801,7 +805,7 @@ int UI::handle(int event)
 // API for the User Interface
 //----------------------------
 
-/*
+/**
  * Get the text from the location input-box.
  */
 const char *UI::get_location()
@@ -809,7 +813,7 @@ const char *UI::get_location()
    return Location->value();
 }
 
-/*
+/**
  * Set a new URL in the location input-box.
  */
 void UI::set_location(const char *str)
@@ -819,7 +823,7 @@ void UI::set_location(const char *str)
    Location->position((Fl::focus() == Location) ? strlen(str) : 0);
 }
 
-/*
+/**
  * Focus location entry.
  * If it's not visible, show it until the callback is done.
  */
@@ -834,7 +838,7 @@ void UI::focus_location()
    Location->position(Location->size(), 0);
 }
 
-/*
+/**
  * Focus Main area.
  */
 void UI::focus_main()
@@ -842,7 +846,7 @@ void UI::focus_main()
    Main->take_focus();
 }
 
-/*
+/**
  * Set a new message in the status bar.
  */
 void UI::set_status(const char *str)
@@ -850,8 +854,8 @@ void UI::set_status(const char *str)
    StatusOutput->value(str);
 }
 
-/*
- * Set the page progress text
+/**
+ * Set the page progress text.
  * cmd: 0 Deactivate, 1 Update, 2 Clear
  */
 void UI::set_page_prog(size_t nbytes, int cmd)
@@ -882,8 +886,8 @@ void UI::set_page_prog(size_t nbytes, int cmd)
    }
 }
 
-/*
- * Set the image progress text
+/**
+ * Set the image progress text.
  * cmd: 0 Deactivate, 1 Update, 2 Clear
  */
 void UI::set_img_prog(int n_img, int t_img, int cmd)
@@ -904,7 +908,7 @@ void UI::set_img_prog(int n_img, int t_img, int cmd)
    }
 }
 
-/*
+/**
  * Set the bug meter progress text
  */
 void UI::set_bug_prog(int n_bug)
@@ -926,7 +930,7 @@ void UI::set_bug_prog(int n_bug)
    StatusBar->rearrange();
 }
 
-/*
+/**
  * Customize the UI's panel (show/hide buttons)
  */
 void UI::customize()
@@ -966,7 +970,7 @@ void UI::customize()
       LocBar->rearrange();
 }
 
-/*
+/**
  * On-the-fly panel style change
  */
 void UI::change_panel(int new_size, int small_icons)
@@ -997,7 +1001,7 @@ void UI::change_panel(int new_size, int small_icons)
    dFree(loc_text);
 }
 
-/*
+/**
  * Set 'nw' as the main render area widget
  */
 void UI::set_render_layout(Fl_Group *nw)
@@ -1012,7 +1016,7 @@ void UI::set_render_layout(Fl_Group *nw)
    TopGroup->resizable(Main);
 }
 
-/*
+/**
  * Set button sensitivity (Back/Forw/Stop)
  */
 void UI::button_set_sens(UIButton btn, int sens)
@@ -1032,7 +1036,7 @@ void UI::button_set_sens(UIButton btn, int sens)
    }
 }
 
-/*
+/**
  * Paste a middle-click-selection into "Clear" button as URL
  */
 void UI::paste_url()
@@ -1040,7 +1044,7 @@ void UI::paste_url()
    Fl::paste(*Clear, false);
 }
 
-/*
+/**
  * Adjust space for the findbar (if necessary) and show or remove it
  */
 void UI::findbar_toggle(bool add)
@@ -1068,7 +1072,7 @@ void UI::findbar_toggle(bool add)
    TopGroup->rearrange();
 }
 
-/*
+/**
  * Make panels disappear growing the render area.
  * WORKAROUND: here we avoid hidden widgets resize by setting their
  *             size to (0,0) while hidden.
