@@ -2,6 +2,7 @@
  * File: uicmd.cc
  *
  * Copyright (C) 2005-2011 Jorge Arellano Cid <jcid@dillo.org>
+ * Copyright (C) 2024 Rodrigo Arias Mallo <rodarima@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -9,8 +10,10 @@
  * (at your option) any later version.
  */
 
-// Functions/Methods for commands triggered from the UI
-
+/**
+ * @file
+ * Functions/Methods for commands triggered from the UI
+ */
 
 #include <string.h>
 #include <stdio.h>
@@ -79,7 +82,7 @@ static void UIcmd_set_window_labels(Fl_Window *win, const char *str);
  * CustTabs ---------------------------------------------------------------
  */
 
-/*
+/**
  * stores the respective UI pointer
  */
 class CustTabButton : public Fl_Button {
@@ -101,7 +104,7 @@ static int btn_cmp(const void *p1, const void *p2)
            (*(CustTabButton * const *)p2)->focus_num() );
 }
 
-/*
+/**
  * Allows fine control of the tabbed interface
  */
 class CustTabs : public Fl_Group {
@@ -181,7 +184,7 @@ public:
    void set_tab_label(UI *ui, const char *title);
 };
 
-/*
+/**
  * Callback for mouse click
  */
 static void tab_btn_cb (Fl_Widget *w, void *cb_data)
@@ -199,7 +202,7 @@ static void tab_btn_cb (Fl_Widget *w, void *cb_data)
    }
 }
 
-/*
+/**
  * Callback for the close-tab button
  */
 static void close_tab_btn_cb (Fl_Widget *, void *cb_data)
@@ -245,12 +248,27 @@ int CustTabs::handle(int e)
          a_Timeout_add(0.0, a_UIcmd_close_all_bw, NULL);
          ret = 1;
       }
+   } else if (e == FL_MOUSEWHEEL && prefs.scroll_switches_tabs) {
+      /* Move to the next or previous tab using the mouse wheel */
+      int dx = Fl::event_dx();
+      int dy = Fl::event_dy();
+      int dir = dy ? dy : dx;
+
+      if (prefs.scroll_switches_tabs_reverse)
+         dir = -dir;
+
+      if (dir > 0)
+         next_tab();
+      else if (dir < 0)
+         prev_tab();
+
+      ret = 1;
    }
 
    return (ret) ? ret : Fl_Group::handle(e);
 }
 
-/*
+/**
  * Create a new tab with its own UI
  */
 UI *CustTabs::add_new_tab(UI *old_ui, int focus)
@@ -307,7 +325,7 @@ UI *CustTabs::add_new_tab(UI *old_ui, int focus)
    return new_ui;
 }
 
-/*
+/**
  * Remove tab by UI
  */
 void CustTabs::remove_tab(UI *ui)
@@ -364,8 +382,8 @@ int CustTabs::get_btn_idx(UI *ui)
    return -1;
 }
 
-/*
- * Keep active tab visible
+/**
+ * Keep active tab visible.
  * (Pack children have unusable x() coordinate)
  */
 void CustTabs::update_pack_offset()
@@ -397,8 +415,8 @@ void CustTabs::update_pack_offset()
    _MSG(" >>scr_x=%d btn0_x=%d\n", scr_x, Pack->child(0)->x());
 }
 
-/*
- * Make cbtn's tab the active one
+/**
+ * Make cbtn's tab the active one.
  */
 void CustTabs::switch_tab(CustTabButton *cbtn)
 {
@@ -458,8 +476,8 @@ void CustTabs::next_tab()
       switch_tab((CustTabButton*)Pack->child((idx+1<num_tabs()) ? idx+1 : 0));
 }
 
-/*
- * Set this UI's tab button label
+/**
+ * Set this UI's tab button label.
  */
 void CustTabs::set_tab_label(UI *ui, const char *label)
 {
@@ -550,7 +568,7 @@ BrowserWindow *a_UIcmd_browser_window_new(int ww, int wh,
       win = new Fl_Double_Window(ww, wh);
 
    win->box(FL_NO_BOX);
-   CustTabs *DilloTabs = new CustTabs(ww, wh, 16);
+   CustTabs *DilloTabs = new CustTabs(ww, wh, prefs.ui_tab_height);
    win->end();
    win->resizable(DilloTabs->wizard());
 
@@ -607,7 +625,7 @@ static BrowserWindow *UIcmd_tab_new(CustTabs *tabs, UI *old_ui, int focus)
    viewport->setDragScroll (prefs.middle_click_drags_page ? true : false);
    layout->attachView (viewport);
    new_ui->set_render_layout(viewport);
-   viewport->setScrollStep((int) rint(28.0 * prefs.font_factor));
+   viewport->setScrollStep(prefs.scroll_step);
 
    // Now, create a new browser window structure
    BrowserWindow *new_bw = a_Bw_new();
@@ -717,8 +735,6 @@ void a_UIcmd_open_urlstr(void *vbw, const char *urlstr)
          ch = new_urlstr[5];
          if (!ch || ch == '.') {
             url = a_Url_new(Paths::getOldWorkingDir(), "file:");
-         } else if (ch == '~') {
-            url = a_Url_new(dGethomedir(), "file:");
          } else {
             url = a_Url_new(new_urlstr, "file:");
          }
@@ -933,7 +949,7 @@ static int UIcmd_save_file_check(const char *name)
       int ch;
       ds = dStr_sized_new(128);
       dStr_sprintf(ds,
-                  "The file:\n  %s (%d Bytes)\nalready exists. What do we do?",
+                  "The file: %s (%d Bytes) already exists. What do we do?",
                    name, (int)ss.st_size);
       ch = a_Dialog_choice("Dillo Save: File exists!", ds->str,
                            "Abort", "Continue", "Rename", NULL);
@@ -1229,7 +1245,7 @@ void a_UIcmd_view_page_bugs(void *vbw)
    if (bw->num_page_bugs > 0) {
       a_Dialog_text_window("Dillo: Detected HTML errors", bw->page_bugs->str);
    } else {
-      a_Dialog_msg("Dillo: Valid HTML!", "Zero detected HTML errors!");
+      a_Dialog_msg("Dillo: Good HTML!", "No HTML errors found while parsing!");
    }
 }
 
