@@ -9,7 +9,7 @@
  * (at your option) any later version.
  */
 
-/*
+/** @file
  * Parse and normalize all URL's inside Dillo.
  *  - <scheme> <authority> <path> <query> and <fragment> point to 'buffer'.
  *  - 'url_string' is built upon demand (transparent to the caller).
@@ -46,6 +46,7 @@
 #include <ctype.h>
 
 #include "url.h"
+#include "hsts.h"
 #include "msg.h"
 
 static const char *HEX = "0123456789ABCDEF";
@@ -56,7 +57,7 @@ static const char *HEX = "0123456789ABCDEF";
 #define URL_STR_FIELD_I_CMP(s1,s2) \
    (s1) && (s2) ? dStrAsciiCasecmp(s1,s2) : !(s1) && !(s2) ? 0 : (s1) ? 1 : -1
 
-/*
+/**
  * Return the url as a string.
  * (initializing 'url_string' field if necessary)
  */
@@ -87,7 +88,7 @@ char *a_Url_str(const DilloUrl *u)
    return url->url_string->str;
 }
 
-/*
+/**
  * Return the hostname as a string.
  * (initializing 'hostname' and 'port' fields if necessary)
  * Note: a similar approach can be taken for user:password auth.
@@ -118,10 +119,16 @@ const char *a_Url_hostname(const DilloUrl *u)
       }
    }
 
+   if (!url->port) {
+      if (!dStrAsciiCasecmp(url->scheme, "http"))
+         url->port = URL_HTTP_PORT;
+      else if (!dStrAsciiCasecmp(url->scheme, "https"))
+         url->port = URL_HTTPS_PORT;
+   }
    return url->hostname;
 }
 
-/*
+/**
  *  Create a DilloUrl object and initialize it.
  *  (buffer, scheme, authority, path, query and fragment).
  */
@@ -134,10 +141,17 @@ static DilloUrl *Url_object_new(const char *uri_str)
 
    url = dNew0(DilloUrl, 1);
 
+   /* url->buffer is given a little extra room in case HSTS needs to transform
+    * a URL string ending in ":80" to ":443".
+    */
+   int len = strlen(uri_str)+2;
+   s = dNew(char, len);
+   memcpy(s, uri_str, len-1);
+   s = dStrstrip(s);
+
    /* remove leading & trailing space from buffer */
-   url->buffer = dStrstrip(dStrdup(uri_str));
+   url->buffer = s;
 
-   s = (char *) url->buffer;
    p = strpbrk(s, ":/?#");
    if (p && p[0] == ':' && p > s) {                /* scheme */
       *p = 0;
@@ -185,8 +199,8 @@ static DilloUrl *Url_object_new(const char *uri_str)
    return url;
 }
 
-/*
- *  Free a DilloUrl
+/**
+ *  Free a DilloUrl.
  *  Do nothing if the argument is NULL
  */
 void a_Url_free(DilloUrl *url)
@@ -198,16 +212,14 @@ void a_Url_free(DilloUrl *url)
          dFree((char *)url->hostname);
       dFree((char *)url->buffer);
       dStr_free(url->data, 1);
-      dFree((char *)url->alt);
       dFree(url);
    }
 }
 
-/*
+/**
  * Resolve the URL as RFC3986 suggests.
  */
 static Dstr *Url_resolve_relative(const char *RelStr,
-                                  DilloUrl *BaseUrlPar,
                                   const char *BaseStr)
 {
    char *p, *s, *e;
@@ -218,9 +230,7 @@ static Dstr *Url_resolve_relative(const char *RelStr,
    /* parse relative URL */
    RelUrl = Url_object_new(RelStr);
 
-   if (BaseUrlPar) {
-      BaseUrl = BaseUrlPar;
-   } else if (RelUrl->scheme == NULL) {
+   if (RelUrl->scheme == NULL) {
       /* only required when there's no <scheme> in RelStr */
       BaseUrl = Url_object_new(BaseStr);
    }
@@ -330,15 +340,15 @@ static Dstr *Url_resolve_relative(const char *RelStr,
 done:
    dStr_free(Path, TRUE);
    a_Url_free(RelUrl);
-   if (BaseUrl != BaseUrlPar)
-      a_Url_free(BaseUrl);
+   a_Url_free(BaseUrl);
    return SolvedUrl;
 }
 
-/*
+/**
  *  Transform (and resolve) an URL string into the respective DilloURL.
  *  If URL  =  "http://dillo.sf.net:8080/index.html?long#part2"
  *  then the resulting DilloURL should be:
+ *  @code
  *  DilloURL = {
  *     url_string         = "http://dillo.sf.net:8080/index.html?long#part2"
  *     scheme             = "http"
@@ -350,9 +360,9 @@ done:
  *     port               = 8080
  *     flags              = URL_Get
  *     data               = Dstr * ("")
- *     alt                = NULL
  *     ismap_url_len      = 0
  *  }
+ *  @endcode
  *
  *  Return NULL if URL is badly formed.
  */
@@ -379,10 +389,10 @@ DilloUrl* a_Url_new(const char *url_str, const char *base_url)
       for (i = 0; url_str[i]; ++i)
          if (url_str[i] > 0x1F && url_str[i] < 0x7F && url_str[i] != ' ')
             *p++ = url_str[i];
-         else  {
-           *p++ = '%';
-           *p++ = HEX[(url_str[i] >> 4) & 15];
-           *p++ = HEX[url_str[i] & 15];
+         else {
+            *p++ = '%';
+            *p++ = HEX[(url_str[i] >> 4) & 15];
+            *p++ = HEX[url_str[i] & 15];
          }
       *p = 0;
       urlstr = str1;
@@ -400,7 +410,7 @@ DilloUrl* a_Url_new(const char *url_str, const char *base_url)
    }
 
    /* Resolve the URL */
-   SolvedUrl = Url_resolve_relative(urlstr, NULL, base_url);
+   SolvedUrl = Url_resolve_relative(urlstr, base_url);
    _MSG("SolvedUrl = %s\n", SolvedUrl->str);
 
    /* Fill url data */
@@ -412,11 +422,48 @@ DilloUrl* a_Url_new(const char *url_str, const char *base_url)
 
    dFree(str1);
    dFree(str2);
+
+   bool_t switch_to_https = FALSE;
+
+   if (url->scheme && !dStrAsciiCasecmp(url->scheme, "http")) {
+      /*
+       * A site's HTTP Strict Transport Security policy may direct us to transform
+       * URLs like "http://en.wikipedia.org:80" to "https://en.wikipedia.org:443".
+       */
+      if (prefs.http_strict_transport_security &&
+          a_Hsts_require_https(a_Url_hostname(url))) {
+         _MSG("url: HSTS transformation for %s.\n", url->url_string->str);
+         switch_to_https = TRUE;
+      } else if (prefs.http_force_https) {
+         _MSG("url: Force HTTPS transformation for %s.\n", url->url_string->str);
+         switch_to_https = TRUE;
+      }
+   }
+
+   if (switch_to_https) {
+      const char *const scheme = "https";
+
+      url->scheme = scheme;
+      if (url->port == URL_HTTP_PORT)
+         url->port = URL_HTTPS_PORT;
+
+      if (url->authority) {
+         int len = strlen(url->authority);
+
+         if (len >= 3 && !strcmp(url->authority + len-3, ":80")) {
+            strcpy((char *)url->authority + len-2, "443");
+         }
+      }
+      
+      dStr_free(url->url_string, TRUE);
+      url->url_string = NULL;
+   }
+
    return url;
 }
 
 
-/*
+/**
  *  Duplicate a Url structure
  */
 DilloUrl* a_Url_dup(const DilloUrl *ori)
@@ -429,7 +476,6 @@ DilloUrl* a_Url_dup(const DilloUrl *ori)
    url->url_string           = dStr_new(URL_STR(ori));
    url->port                 = ori->port;
    url->flags                = ori->flags;
-   url->alt                  = dStrdup(ori->alt);
    url->ismap_url_len        = ori->ismap_url_len;
    url->illegal_chars        = ori->illegal_chars;
    url->illegal_chars_spc    = ori->illegal_chars_spc;
@@ -438,7 +484,7 @@ DilloUrl* a_Url_dup(const DilloUrl *ori)
    return url;
 }
 
-/*
+/**
  *  Compare two Url's to check if they're the same, or which one is bigger.
  *
  *  The fields which are compared here are:
@@ -467,7 +513,7 @@ int a_Url_cmp(const DilloUrl *A, const DilloUrl *B)
    return st;
 }
 
-/*
+/**
  * Set DilloUrl flags
  */
 void a_Url_set_flags(DilloUrl *u, int flags)
@@ -476,7 +522,7 @@ void a_Url_set_flags(DilloUrl *u, int flags)
       u->flags = flags;
 }
 
-/*
+/**
  * Set DilloUrl data (like POST info, etc.)
  */
 void a_Url_set_data(DilloUrl *u, Dstr **data)
@@ -488,19 +534,8 @@ void a_Url_set_data(DilloUrl *u, Dstr **data)
    }
 }
 
-/*
- * Set DilloUrl alt (alternate text to the URL. Used by image maps)
- */
-void a_Url_set_alt(DilloUrl *u, const char *alt)
-{
-   if (u) {
-      dFree((char *)u->alt);
-      u->alt = dStrdup(alt);
-   }
-}
-
-/*
- * Set DilloUrl ismap coordinates
+/**
+ * Set DilloUrl ismap coordinates.
  * (this is optimized for not hogging the CPU)
  */
 void a_Url_set_ismap_coords(DilloUrl *u, char *coord_str)
@@ -509,8 +544,7 @@ void a_Url_set_ismap_coords(DilloUrl *u, char *coord_str)
 
    if (!u->ismap_url_len) {
       /* Save base-url length (without coords) */
-      u->ismap_url_len  = URL_STR_(u) ? u->url_string->len : 0;
-      a_Url_set_flags(u, URL_FLAGS(u) | URL_Ismap);
+      u->ismap_url_len = URL_STR_(u) ? u->url_string->len : 0;
    }
    if (u->url_string) {
       dStr_truncate(u->url_string, u->ismap_url_len);
@@ -519,7 +553,7 @@ void a_Url_set_ismap_coords(DilloUrl *u, char *coord_str)
    }
 }
 
-/*
+/**
  * Given an hex octet (e.g., e3, 2F, 20), return the corresponding
  * character if the octet is valid, and -1 otherwise
  */
@@ -537,7 +571,7 @@ static int Url_decode_hex_octet(const char *s)
    return -1;
 }
 
-/*
+/**
  * Parse possible hexadecimal octets in the URI path.
  * Returns a new allocated string.
  */
@@ -565,8 +599,8 @@ char *a_Url_decode_hex_str(const char *str)
    return new_str;
 }
 
-/*
- * Urlencode 'str'
+/**
+ * Urlencode 'str'.
  * -RL :: According to the RFC 1738, only alphanumerics, the special
  *        characters "$-_.+!*'(),", and reserved characters ";/?:@=&" used
  *        for their *reserved purposes* may be used unencoded within a URL.
@@ -608,7 +642,7 @@ char *a_Url_encode_hex_str(const char *str)
 }
 
 
-/*
+/**
  * RFC-3986 suggests this stripping when "importing" URLs from other media.
  * Strip: "URL:", enclosing < >, and embedded whitespace.
  * (We also strip illegal chars: 00-1F and 7F-FF)
@@ -635,33 +669,31 @@ char *a_Url_string_strip_delimiters(const char *str)
    return new_str;
 }
 
-/*
- * Is the provided hostname an IP address?
+/**
+ * What type of host is this?
  */
-static bool_t Url_host_is_ip(const char *host)
+int a_Url_host_type(const char *host)
 {
    uint_t len;
 
    if (!host || !*host)
-      return FALSE;
+      return URL_HOST_ERROR;
 
    len = strlen(host);
 
    if (len == strspn(host, "0123456789.")) {
-      _MSG("an IPv4 address\n");
-      return TRUE;
+      return URL_HOST_IPV4;
    }
    if (strchr(host, ':') &&
        (len == strspn(host, "0123456789abcdefABCDEF:."))) {
       /* The precise format is shown in section 3.2.2 of rfc 3986 */
-      MSG("an IPv6 address\n");
-      return TRUE;
+      return URL_HOST_IPV6;
    }
-   return FALSE;
+   return URL_HOST_NAME;
 }
 
-/*
- * How many internal dots are in the public portion of this hostname?
+/**
+ * How many internal dots are in the public portion of this hostname?.
  * e.g., for "www.dillo.org", it is one because everything under "dillo.org",
  * as a .org domain, is part of one organization.
  *
@@ -688,7 +720,7 @@ static uint_t Url_host_public_internal_dots(const char *host)
 
       if (tld_len > 0) {
          /* These TLDs were chosen by examining the current publicsuffix list
-          * in October 2014 and picking out those where it was simplest for
+          * in July 2016 and picking out those where it was simplest for
           * them to describe the situation by beginning with a "*.[tld]" rule
           * or every rule was "[something].[tld]".
           *
@@ -697,8 +729,8 @@ static uint_t Url_host_public_internal_dots(const char *host)
           * poorer approximation of administrative boundaries.
           */
          const char *const tlds[] = {"bd","bn","ck","cy","er","fj","fk",
-                                     "gu","il","jm","ke","kh","kw","mm","mz",
-                                     "ni","np","pg","ye","za","zm","zw"};
+                                     "gu","jm","ke","kh","kw","mm","mz",
+                                     "ni","np","pg","ye","za","zw"};
          uint_t i, tld_num = sizeof(tlds) / sizeof(tlds[0]);
 
          for (i = 0; i < tld_num; i++) {
@@ -714,8 +746,8 @@ static uint_t Url_host_public_internal_dots(const char *host)
    return ret;
 }
 
-/*
- * Given a URL host string, return the portion that is public, i.e., the
+/**
+ * Given a URL host string, return the portion that is public. i.e., the
  * domain that is in a registry outside the organization.
  * For 'www.dillo.org', that would be 'dillo.org'.
  */
@@ -724,7 +756,7 @@ static const char *Url_host_find_public_suffix(const char *host)
    const char *s;
    uint_t dots;
 
-   if (!host || !*host || Url_host_is_ip(host))
+   if (a_Url_host_type(host) != URL_HOST_NAME)
       return host;
 
    s = host;
