@@ -3,6 +3,9 @@
  *
  * Copyright (C) 2001 Jorge Arellano Cid <jcid@dillo.org>
  *
+ */
+
+/** @file
  * Parse and normalize all URL's inside Dillo.
  */
 
@@ -13,14 +16,14 @@
 #include "../dlib/dlib.h"
 
 
-#define DILLO_URL_HTTP_PORT        80
-#define DILLO_URL_HTTPS_PORT       443
-#define DILLO_URL_FTP_PORT         21
-#define DILLO_URL_MAILTO_PORT      25
-#define DILLO_URL_NEWS_PORT        119
-#define DILLO_URL_TELNET_PORT      23
-#define DILLO_URL_GOPHER_PORT      70
+#define URL_HTTP_PORT        80
+#define URL_HTTPS_PORT       443
 
+/* for a_Url_host_type() */
+#define URL_HOST_ERROR      -1
+#define URL_HOST_NAME        0
+#define URL_HOST_IPV4        1
+#define URL_HOST_IPV6        2
 
 /*
  * Values for DilloUrl->flags.
@@ -28,12 +31,8 @@
  */
 #define URL_Get                 (1 << 0)
 #define URL_Post                (1 << 1)
-#define URL_ISindex             (1 << 2)
-#define URL_Ismap               (1 << 3)
-#define URL_RealmAccess         (1 << 4)
 
 #define URL_E2EQuery            (1 << 5)
-#define URL_ReloadImages        (1 << 6)
 #define URL_ReloadPage          (1 << 7)
 #define URL_ReloadFromCache     (1 << 8)
 
@@ -53,7 +52,6 @@
 #define URL_QUERY_(u)               (u)->query
 #define URL_FRAGMENT_(u)            (u)->fragment
 #define URL_HOST_(u)                a_Url_hostname(u)
-#define URL_ALT_(u)                 (u)->alt
 #define URL_STR_(u)                 a_Url_str(u)
 /* this returns a Dstr* */
 #define URL_DATA_(u)                (u)->data
@@ -75,9 +73,8 @@
 #define URL_QUERY(u)                NPTR2STR(URL_QUERY_(u))
 #define URL_FRAGMENT(u)             NPTR2STR(URL_FRAGMENT_(u))
 #define URL_HOST(u)                 NPTR2STR(URL_HOST_(u))
-#define URL_DATA(u)                 URL_DATA_(u)
-#define URL_ALT(u)                  NPTR2STR(URL_ALT_(u))
 #define URL_STR(u)                  NPTR2STR(URL_STR_(u))
+#define URL_DATA(u)                 URL_DATA_(u)
 #define URL_PORT(u)                 URL_PORT_(u)
 #define URL_FLAGS(u)                URL_FLAGS_(u)
 #define URL_ILLEGAL_CHARS(u)        URL_ILLEGAL_CHARS_(u)
@@ -99,11 +96,10 @@ typedef struct {
    const char *hostname;          /**/
    int port;
    int flags;
-   Dstr *data;                    /* POST */
-   const char *alt;               /* "alt" text (used by image maps) */
-   int ismap_url_len;             /* Used by server side image maps */
-   int illegal_chars;             /* number of illegal chars */
-   int illegal_chars_spc;         /* number of illegal space chars */
+   Dstr *data;                    /**< POST */
+   int ismap_url_len;             /**< Used by server side image maps */
+   int illegal_chars;             /**< number of illegal chars */
+   int illegal_chars_spc;         /**< number of illegal space chars */
 } DilloUrl;
 
 
@@ -115,11 +111,11 @@ DilloUrl* a_Url_dup(const DilloUrl *u);
 int a_Url_cmp(const DilloUrl *A, const DilloUrl *B);
 void a_Url_set_flags(DilloUrl *u, int flags);
 void a_Url_set_data(DilloUrl *u, Dstr **data);
-void a_Url_set_alt(DilloUrl *u, const char *alt);
 void a_Url_set_ismap_coords(DilloUrl *u, char *coord_str);
 char *a_Url_decode_hex_str(const char *str);
 char *a_Url_encode_hex_str(const char *str);
 char *a_Url_string_strip_delimiters(const char *str);
+int a_Url_host_type(const char *host);
 bool_t a_Url_same_organization(const DilloUrl *u1, const DilloUrl *u2);
 #ifdef __cplusplus
 }
